#include <iostream>
#include <string>
#include <fstream>
#include <algorithm>
#include <memory>
#include "NeuMeshLoader.h"
#include "MeshLoader.h"
#include "AneuMeshLoader.h"


MeshLoader* File_Format_Checker(const std::string& cfile)//   ,     file
{ 
	std::string file = cfile;
    while (file.back() == ' ')                     
        file.pop_back();
    MeshLoader* mesh_pointer = nullptr;

    // check .aneu
    std::size_t found = file.rfind(".aneu");
    if (found != std::string::npos && file.length() - found == 5) 
	{
        std::cout << "File-format-checker was passed." << std::endl;
        //dynamic_cast<AneuMeshLoader*>(mesh_pointer);
        mesh_pointer = new AneuMeshLoader;
        return mesh_pointer;
    }

    // check .neu
    found = file.rfind(".neu");
    if (found != std::string::npos && file.length() - found == 4) 
	{
        std::cout << "File-format-checker was passed." << std::endl;
        //dynamic_cast<NeuMeshLoader*>(mesh_pointer);   
        mesh_pointer = new NeuMeshLoader;//smart 
        return mesh_pointer;
    }

    else 
	{
        std::cout << "Warning! File-format-checker was not passed!" << std::endl;
        return 0;
        //std::size_t dot = file.rfind('.');
        /*if (dot == std::string::npos)
            throw BadFileFormatException();
        else throw BadFileFormatException(file.substr(file.rfind('.'), file.size() - file.rfind('.')));*/
    };
}

void Mesh_Loader_Demo(MeshLoader* loader_, const std::string& filepath_) 
{
    loader_->LoadMesh(filepath_);
    loader_->Print_Data();
    std::vector<FiniteElement> elements_vector;
    std::vector<Node> nodes_vector;
    std::map<int,std::vector<Node>> neighbours;
    std::vector<BoundaryFiniteElement> boundaryFEs_vector;

    std::cout << "Result of Get_Elements_by_ID(2, 4, 8) function:\n";
    elements_vector = loader_->getElementsByID(2, 4, 8);
    std::cout << elements_vector;
    elements_vector.clear();

    std::cout << "\nResult of Get_Elements_by_edge(2, 8) function:\n";
    elements_vector = loader_->getElementsByEdge(2, 8);
    std::cout << elements_vector;
    elements_vector.clear();

    std::cout << "\nResult of Get_Vertex_Nodes_by_Surface_ID(6) function:\n";
    nodes_vector = loader_->getNodesByBoundary(6);
    std::cout << nodes_vector;
    nodes_vector.clear();

    std::cout << "\nResult of Get_Elements_by_Material_ID(1) function:\n";
    elements_vector = loader_->getElementsByBoundary(1);
    std::cout << elements_vector;
    elements_vector.clear();

    std::cout << "\nResult of Get_boundaryFEs_by_Surface_ID(1) function:\n";
    boundaryFEs_vector = loader_->getsboundaryFEsByBoundary(1);
    std::cout << boundaryFEs_vector;
    boundaryFEs_vector.clear();

	auto N = loader_->getNodes();
    std::cout << "\nResult of Get_Nodes_Neighbors() function:\n";
	for (const auto& i : N) 
	{
		std::cout << "For node with ID "<< i.nodeID << " : " << std::endl;
		neighbours = loader_->getNodeNeighbours();
		for (int j = 1; j < neighbours.size(); ++j)
			std::cout << "Neighbors for Node with ID = " << neighbours[j] << std::endl;
	}
    neighbours.clear();

	std::cout << "\nResult of insertCenterNode() function:\n";
	loader_->insertCenterNode();
	loader_->Print_Data();
}

struct fixed_tolower : std::unary_function<char, char> 
{ 
    char operator()(char c) const 
	{
        return tolower((unsigned char)c);
    }
};


int main(int argc, char* argv[]) 
{
    std::string filepath = "C:\\Users\\Saray\\Desktop\\MeshLoader\\cube.aneu";
    if (argc == 1)
        std::cout << "No arguments were added, using standard file PATH..." << std::endl;
    else if (argc > 2)
        std::cout << "There must be one or none arguments! Set file path to default..." << std::endl;
    else 
	{
        std::cout << "Set file path to \"" << argv[1] << "\"..." << std::endl;
        filepath = argv[1];
        std::transform(filepath.begin(), filepath.end(), filepath.begin(), fixed_tolower()); // .aneu = .ANEU = .AnEu = etc
    };

    MeshLoader* loader = File_Format_Checker(filepath);
    Mesh_Loader_Demo(loader, filepath);
	delete loader;
    return 0;
}